from time import sleep
from picamera import PiCamera
import smbus

#To adapt a value of the commands below, please only change the value indicated by the letter "V",
#for example if AWBRgainUser is defined as 0xVV11, so can set the value to 0x0011, or 0xFF11 or any value in between

DEVICE_ADDR = 0x1e
REG_ADDR_CAMCOMMANDS = 0xf2 #commands for zoom etc.
REG_ADDR_BOARDCOMMANDS = 0x00 #commands for ISP of camera
DATA0 = 0x00
ZoomStop = 0x0000 #stop zoom
ZoomTeleFast = 0x7100 #zoom Tele command = 0xV100
ZoomTeleAverage =0x4100
ZoomTeleSlow = 0x1100
ZoomWideFast = 0x7200 #zoom Wide command = 0xV200
ZoomWideAverage =0x4200
ZoomWideSlow = 0x1200
ZoomPositionExecute = 0x0300
AWBAuto = 0x0010
AWBPushToSetStep1 = 0x0210 #set camera to AWB Push mode
AWBPushToSetStep2 = 0xA514 #execute AWB once
AWBManual = 0x0110
ImageMirrorOff = 0x0030
ImageMirrorOn = 0x0130
ImageFlipOff = 0x0031
ImageFlipOn = 0x0131
WDROff = 0x8024
WDROn = 0x0024
DNROff = 0x0025
DNROn = 0x8025
Gamma045 = 0x0026 
Gamma055 = 0x0126
Gamma065 = 0x0226
Gamma075 = 0x0326
FocusAuto = 0xFF04
FocusOnZoom = 0x8004
FocusManual = 0xc004
FocusFarSlow = 0x0104
FocusFarFast = 0x7104
FocusNearSlow = 0x0204
FocusNearFast = 0x7204
FocusStop = 0x0004
AEXAuto = 0x0020 #auto exposure and gain
AEXManual = 0x0120 #manual exposure and gain
AEXDNmodeDay = 0x012A
AEXDNmodeNight = 0x022A
AEXDNmodeAuto = 0x002A
VideoMode720p60Cam = 0x0033 #does not work yet 
VideoMode720p50Cam = 0x0233 #does not work yet 
VideoMode1080p30Cam = 0x0133
VideoMode1080p25Cam = 0x0333
VideoMode1080pPi = (1920,1080) #does not work yet 
VideoMode720pPi = (1280,720) #does not work yet 

bus = smbus.SMBus(1)
camera = PiCamera()
shutter = camera.shutter_speed #read shutter speed
camera.shutter_speed = shutter*2 #set pi camera shutter speed
camera.awb_mode = 'off' #disable AWB of Pi
camera.awb_gains = 1.34, 1.44 #set red gain and blue gain of AWB of Pi to fixed value
camera.exposure_mode = 'off' #disable auto exposure pi communication




camera.start_preview()

#Set saturation of camera
SaturationB = 0x9050 #SaturationB command = 0xVV50
SaturationY = 0x9051 #SaturationY command = 0xVV51
SaturationR = 0x9052 #SaturationR command = 0xVV52
SaturationG = 0x9053 #SaturationG command = 0xVV53
bus.write_word_data(DEVICE_ADDR, 0x00, SaturationB)
bus.write_word_data(DEVICE_ADDR, 0x00, SaturationY)
bus.write_word_data(DEVICE_ADDR, 0x00, SaturationR)
bus.write_word_data(DEVICE_ADDR, 0x00, SaturationG)

#Start and stop focus
bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, FocusManual)
sleep(1)
bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, FocusFarSlow)
sleep(3)
bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, FocusNearSlow)
sleep(3)
bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, FocusFarFast)
sleep(3)
bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, FocusNearFast)
sleep(3)
bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, FocusStop)

sleep(3)
camera.stop_preview()

#Set saturation of camera
#SaturationB = 0x9050 #SaturationB command = 0xVV50
#SaturationY = 0x9051 #SaturationY command = 0xVV51
#SaturationR = 0x9052 #SaturationR command = 0xVV52
#SaturationG = 0x9053 #SaturationG command = 0xVV53
#bus.write_word_data(DEVICE_ADDR, 0x00, SaturationB)
#bus.write_word_data(DEVICE_ADDR, 0x00, SaturationY)
#bus.write_word_data(DEVICE_ADDR, 0x00, SaturationR)
#bus.write_word_data(DEVICE_ADDR, 0x00, SaturationG)

#Set AWB to manual value (to set camera back in AWB auto you need to reboot the raspberry Pi)
#AWBRgainUser = 0x6011 #AWBRgain value = 0xVV11
#AWBBgainUser = 0x1012 #AWBBgain value = 0xVV12
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, AWBManual)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, AWBRgainUser)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, AWBBgainUser)

#Set AWB to auto (this function is not implemented)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_BOARDCOMMANDS, AWBAuto)

#set AWB to once (this function is not implemented)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, AWBPushToSetStep1)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, AWBPushToSetStep2)

#Set day night mode (with or without IR cut filter)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, AEXDNmodeDay)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, AEXDNmodeNight)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, AEXDNmodeAuto)

#save an image (this is not implemented)
#camera.capture('/home/pi/Desktop/picture.jpg') #save as JPG
#camera.capture('/home/pi/Desktop/picture.data', 'rgb') # save as RAW data
#camera.capture('/home/pi/Desktop/picture.bmp', 'bmp') # save as BMP data

#Set gain and exposure to automatic (default setting at startup)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, AEXAuto)

#Set AEX = gain, exposure and iris to manual
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, AEXManual)
#sleep(1)
#AEXManualGain = 0x1021 #gain value = 0xVV21 (Low = 0x0021 High = 0x0a21)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, AEXManualGain)
#sleep(1)
#AEXManualExposure = 0x0922 #exposure value = 0xVV22 (value 0xFF22 = flickerless, Max exposure = 0x0022)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, AEXManualExposure)
#sleep(1)
#AEXManualIris = 0x0123 #iris value = 0xVV23 (almost close = 0x0023 Max open = 0x1123)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, AEXManualIris)
#sleep(1)

#Wide dynamic range
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, WDROff)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, WDROn)
#WDRLevel =0x0028 #level of WDR = 0x0V28 (value between 0-4) #not working
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, WDRLevel) #set the WDR level #not working

#Digital Noise Reduction
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, DNROff)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, DNROn)
#DNRLevel =0x0129 #level of DNR = 0x0V28 (value between 1-3), the higher the value the more noise reduction
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, DNRLevel) #set the DNR level

#Set Gamma
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, Gamma045)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, Gamma055)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, Gamma065)

#Set rotation / mirror image
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, ImageFlipOn)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, ImageFlipOff)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, ImageMirrorOn)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, ImageMirrorOff)

#Start and stop zoom
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, ZoomTeleSlow)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, ZoomTeleAverage)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, ZoomTeleFast)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, ZoomWideSlow)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, ZoomWideAverage)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, ZoomWideFast)
#sleep(1)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, ZoomStop)

#Set zoom to predifined position
#ZoomPositionMSB = 0x0101 #set MSB of zoomposition 0xVV01, max value 0x3F01
#ZoomPositionLSB = 0x0102 #set LSB of zoomposition 0xVV01, max value 0xFF02
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, ZoomPositionMSB)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, ZoomPositionLSB)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, ZoomPositionExecute)

#Start and stop focus
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, FocusManual)
#sleep(1)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, FocusFarSlow)
#sleep(1)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, FocusNearSlow)
#sleep(1)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, FocusFarFast)
#sleep(1)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, FocusNearFast)
#sleep(1)
#bus.write_word_data(DEVICE_ADDR, REG_ADDR_CAMCOMMANDS, FocusStop)
