#include "I2C_interface.h"
#include <iostream>
#include <unistd.h>
#include <sys/ioctl.h>
#include <linux/i2c-dev.h>
#include <fcntl.h>

uint8_t LoadDefaultValuesAtStartUp(uint8_t *pArrayCamVal)
{
    uint8_t errorMessage = SUCCESS;
    *(pArrayCamVal + 0 ) = DEFAULT_ZOOM_POS;
    *(pArrayCamVal + 1 ) = DEFAULT_ZOOM_SPEED ;
    *(pArrayCamVal + 2 ) = DEFAULT_FOCUS_SPEED;
    *(pArrayCamVal + 3 ) = DEFAULT_MANUAL_GAIN;
    *(pArrayCamVal + 4 ) = DEFAULT_MANUAL_SHUTTER ;
    *(pArrayCamVal + 5 ) = DEFAULT_MANUAL_IRIS;
    *(pArrayCamVal + 6 ) = DEFAULT_WDR_LEVEL;
    *(pArrayCamVal + 7 ) = DEFAULT_DNR_LEVEL;
    *(pArrayCamVal + 8 ) = DEFAULT_MANUAL_REDGAIN ;
    *(pArrayCamVal + 9 ) = DEFAULT_MANUAL_BLUEGAIN ;
    *(pArrayCamVal + 10 ) = DEFAULT_IM_STABILIZER ;
    *(pArrayCamVal + 11 ) = DEFAULT_IM_STABILIZER_FILTER ;
    *(pArrayCamVal + 12 ) = DEFAULT_APERATURE_GAIN ;
    *(pArrayCamVal + 13 ) = DEFAULT_VIDEO_MODE ;
    return errorMessage;
}


uint8_t I2C_Zoom_Fuction( uint8_t zoomCommand, uint8_t zoomSpeed, uint8_t *zoomData)
{
    uint8_t errorMessage = SUCCESS;
    uint8_t pRegaddress[2];
    uint8_t pDataBuffer[2];
    uint8_t numbDataBytes = 1;
    if(zoomCommand > 0x0f)
    {
        errorMessage = I2C_WRONG_COMMAND;
        return errorMessage;
    }

    pRegaddress[0] = 0xf2;
    switch (zoomCommand){
        case I2C_ZOOM_IN: pRegaddress[1] = 0x00; pDataBuffer[0] = ((zoomSpeed<<4) &0xf0)| 0x01; break;
        case I2C_ZOOM_OUT: pRegaddress[1] = 0x00; pDataBuffer[0] = ((zoomSpeed<<4) &0xf0)| 0x02;  break;
        case I2C_ZOOM_STOP: pRegaddress[1] = 0x00; pDataBuffer[0] = 0x00; break;
        case I2C_ZOOM_MODE_NORM: pRegaddress[1] = 0x03; pDataBuffer[0] = 0xc0; break;
        case I2C_ZOOM_MODE_DIG_OFF: pRegaddress[1] = 0x03; pDataBuffer[0] = *zoomData & 0x3f; break;
        case I2C_ZOOM_MODE_ZOOM_2_POS: pRegaddress[1] = 0x00; pDataBuffer[0] = 0x03; break;
        case I2C_ZOOM_POSITION_H: pRegaddress[1] = 0x01; pDataBuffer[0] = *zoomData;  numbDataBytes = 1; break;
        case I2C_ZOOM_POSITION_L: pRegaddress[1] = 0x02; pDataBuffer[0] = *zoomData;  numbDataBytes = 1; break;
        }

    errorMessage = WriteI2CData(pRegaddress, pDataBuffer, 2,numbDataBytes);
    return errorMessage;
}

uint8_t I2C_Focus_Fuction( uint8_t focusCommand, uint8_t focusSpeed, uint8_t *focusData)
{
    uint8_t errorMessage = SUCCESS;
    uint8_t pRegaddress[2];
    uint8_t pDataBuffer[2];
    uint8_t numbDataBytes = 1;
    pRegaddress[0] = 0xf2;
    if((focusCommand <0x10) || (focusCommand >0x1f))
    {
        errorMessage = I2C_WRONG_COMMAND;
        return errorMessage;
    }
    switch (focusCommand){
        case I2C_FOCUS_AUTO: pRegaddress[1] = 0x04; pDataBuffer[0] = 0xff; break;
        case I2C_FOCUS_ON_ZOOM: pRegaddress[1] = 0x04; pDataBuffer[0] = 0x80; break;
        case I2C_FOCUS_TELE: pRegaddress[1] = 0x04; pDataBuffer[0] = ((focusSpeed<<4) &0xf0)| 0x01; break;
        case I2C_FOCUS_WIDE: pRegaddress[1] = 0x04; pDataBuffer[0] = ((focusSpeed<<4) &0xf0)| 0x02;  break;
        case I2C_FOCUS_STOP: pRegaddress[1] = 0x04; pDataBuffer[0] = 0x00; break;
        case I2C_FOCUS_MAN:  pRegaddress[1] = 0x04; pDataBuffer[0] = 0xc0; break;
        }
    errorMessage = WriteI2CData(pRegaddress, pDataBuffer, 2,numbDataBytes);
    return errorMessage;
}

uint8_t I2C_AWB_Fuction( uint8_t awbCommand,  uint8_t *awbData)
{
    uint8_t errorMessage = SUCCESS;
     uint8_t pRegaddress[2];
    uint8_t pDataBuffer[2];
    uint8_t numbDataBytes = 1;
    pRegaddress[0] = 0xf2;
    if((awbCommand <0x20) || (awbCommand >0x2f))
    {
        errorMessage = I2C_WRONG_COMMAND;
        return errorMessage;
    }
     switch (awbCommand){
        case I2C_AWB_AUTO: pRegaddress[1] = 0x10; pDataBuffer[0] = 0x00; break;
        case I2C_AWB_P2S: pRegaddress[1] = 0x14; pDataBuffer[0] = 0x02; break;
        case AWM_MANUAL_MODE: pRegaddress[1] = 0x10; pDataBuffer[0] = 0x01; break;
        case I2C_WB_RGAIN: pRegaddress[1] = 0x11; pDataBuffer[0] = *awbData; break;
        case I2C_WB_BGAIN: pRegaddress[1] = 0x12; pDataBuffer[0] = *awbData; break;
        case I2C_PSW_ACT: pRegaddress[1] = 0x14; pDataBuffer[0] = 0xa5; break;
        }
    errorMessage = WriteI2CData(pRegaddress, pDataBuffer, 2,numbDataBytes);
    return errorMessage;
}

uint8_t I2C_AEX_Fuction( uint8_t aexCommand, uint8_t *aexData)
{
    uint8_t errorMessage = SUCCESS;
    uint8_t pRegaddress[2];
    uint8_t pDataBuffer[2];
    uint8_t numbDataBytes = 1;
    pRegaddress[0] = 0xf2;
    if((aexCommand <0x30) || (aexCommand >0x3f))
    {
        errorMessage = I2C_WRONG_COMMAND;
        return errorMessage;
    }
    switch (aexCommand){
        case 0x30: pRegaddress[1] = 0x20; pDataBuffer[0] = *aexData;break;
        case 0x31: pRegaddress[1] = 0x21; pDataBuffer[0] = *aexData; break;
        case 0x32: pRegaddress[1] = 0x22; pDataBuffer[0] = *aexData; break;
        case 0x34: pRegaddress[1] = 0x23; pDataBuffer[0] = *aexData; break;
        case 0x36: pRegaddress[1] = 0x2a; pDataBuffer[0] = *aexData; break;
        case 0x38: if((*aexData == 0x80) || (*aexData == 0x00)){ pRegaddress[1] = 0x24; pDataBuffer[0] = *aexData; }    //wdr level
                    else{ pRegaddress[1] = 0x28; pDataBuffer[0] = *aexData; } break;
        case 0x39: if((*aexData == 0x80) || (*aexData == 0x00)){ pRegaddress[1] = 0x25; pDataBuffer[0] = *aexData; }    //wdr level
                    else{ pRegaddress[1] = 0x29; pDataBuffer[0] = *aexData; } break;    //dnr level
        case 0x3a: pRegaddress[1] = 0x26; pDataBuffer[0] = *aexData; break;     // gamma
        case 0x3b: pRegaddress[1] = 0x27; pDataBuffer[0] = *aexData; break;     // aperature
        }
    errorMessage = WriteI2CData(pRegaddress, pDataBuffer, 2,numbDataBytes);
    return errorMessage;
}

uint8_t I2C_WDR_Fuction( uint8_t wdrCommand, uint8_t wdrLevel)
{
    uint8_t errorMessage = SUCCESS;
    if(wdrCommand >0x1)
    {
        errorMessage = I2C_WRONG_COMMAND;
        return errorMessage;
    }
    return errorMessage;
}

uint8_t I2C_DNR_Fuction( uint8_t dnrCommand, uint8_t dnrLevel)
{
    uint8_t errorMessage = SUCCESS;
    if(dnrCommand >0x1)
    {
        errorMessage = I2C_WRONG_COMMAND;
        return errorMessage;
    }
    return errorMessage;
}


uint8_t I2C_Feature_Fuction( uint8_t featureCommand, uint8_t *pData)
{
     uint8_t errorMessage = SUCCESS;
    uint8_t pRegaddress[2];
    uint8_t pDataBuffer[2];
    uint8_t numbDataBytes = 1;
    pRegaddress[0] = 0xf2;
    if((featureCommand <0x50) || (featureCommand >0x5f))
    {
        errorMessage = I2C_WRONG_COMMAND;
        return errorMessage;
    }
    pDataBuffer[0] = *pData;
      switch (featureCommand){
        case 0x51: pRegaddress[1] = 0x30;  break;   // mirror
        case 0x52: pRegaddress[1] = 0x31;  break;   // flip
        case 0x53: pRegaddress[1] = 0x32;  break;   // freeze
        case 0x54: pRegaddress[1] = 0x34;  break;   // image stabilizer
        case 0x55: pRegaddress[1] = 0x33;  break;   // video
        case 0x56: pRegaddress[1] = 0x35;  break;   // stab filter
        case 0x57: pRegaddress[1] = 0x36;  break;   // im st range
        }
    errorMessage = WriteI2CData(pRegaddress, pDataBuffer, 2,numbDataBytes);
    return errorMessage;
}


//*************************************************************************************************************************************************************************************************
// I2C bus interface Raspberry Pi
//  I2C bus 1
//*************************************************************************************************************************************************************************************************

int file_i2c;

int deviceAddress = CAMERA_DEV_ADDRES;


uint8_t OpenI2CBus(void)
{
file_i2c =open("/dev/i2c-1", O_RDWR);
if(file_i2c < 0)
{
        //cout <<"Failed to open the I2C bus" ;
        close(file_i2c);
        return I2C_ERROR_FAILED_TO_OPEN_BUS;
    }
    if(ioctl(file_i2c, I2C_SLAVE,deviceAddress ) <0)
    {
        //cout  << "Failed to aquire BUS access ";
        close(file_i2c);
        return I2C_ERROR_FAILED_CONTACT_DEVICE;
    }
    return SUCCESS;
}


uint8_t WriteI2CData(uint8_t *pRegaddress, uint8_t *pDataBuffer, uint8_t sizeReginBytes, uint8_t sizeDatainBytes)
{
    uint8_t buffer[8] = {0};
    int length;
    if((sizeReginBytes ==0) || (sizeReginBytes > 2)) sizeReginBytes = 1;
    if((sizeDatainBytes ==0) || (sizeDatainBytes > 4)) sizeDatainBytes = 1;
    switch(sizeReginBytes){
    case 1: buffer[0] = *pRegaddress;
            switch(sizeDatainBytes){
            case 1: buffer[1] = *pDataBuffer; length = 2; break;
            case 2: buffer[1] = *pDataBuffer;buffer[2] = *(pDataBuffer+1); length = 3;break;
            case 3: buffer[1] = *pDataBuffer;buffer[2] = *(pDataBuffer+1); buffer[3] = *(pDataBuffer+2); length = 4; break;
            case 4: buffer[1] = *pDataBuffer;buffer[2] = *(pDataBuffer+1); buffer[3] = *(pDataBuffer+2);buffer[5] = *(pDataBuffer+3);length = 5; break;
            }
            break;
    case 2:buffer[0] = *pRegaddress; buffer[1] = *(pRegaddress+1);
            switch(sizeDatainBytes){
            case 1: buffer[2] = *pDataBuffer; length = 3; break;
            case 2: buffer[2] = *pDataBuffer;buffer[3] = *(pDataBuffer+1);  length = 4; break;
            case 3: buffer[2] = *pDataBuffer;buffer[3] = *(pDataBuffer+1); buffer[4] = *(pDataBuffer+2); length = 5; break;
            case 4: buffer[2] = *pDataBuffer;buffer[3] = *(pDataBuffer+1); buffer[4] = *(pDataBuffer+2);buffer[5] = *(pDataBuffer+3);  length = 6; break;
            }
            break;
    }
    OpenI2CBus();
    if(write(file_i2c, buffer,length) !=  length)
    {
            //cout << "Failed I2C writing " ;
            close(file_i2c);
            return I2C_ERROR_FAILED_WRITING;
    }
    close(file_i2c);
    return SUCCESS;
}


uint8_t ReadI2CData(uint8_t *pRegaddress, uint8_t *pDataBuffer, uint8_t sizeReginBytes, uint8_t sizeDatainBytes)
{
    uint8_t buffer[8] = {0};
    int length;
    if((sizeReginBytes ==0) || (sizeReginBytes > 2)) sizeReginBytes = 1;
    if((sizeDatainBytes ==0) || (sizeDatainBytes > 4)) sizeDatainBytes = 1;
    switch(sizeReginBytes){
    case 1: buffer[0] = *pRegaddress;
            switch(sizeDatainBytes){
            case 1:  length = 2; break;
            case 2:  length = 3;break;
            case 3:  length = 4; break;
            case 4:  length = 5; break;
            }
            break;
    case 2:buffer[0] = *pRegaddress; buffer[1] = *(pRegaddress+1);
            switch(sizeDatainBytes){
            case 1: length = 3; break;
            case 2: length = 4; break;
            case 3: length = 5; break;
            case 4: length = 6; break;
            }
            break;
    }
    OpenI2CBus();
    if(read(file_i2c, buffer,length) !=  length)
    {
            //cout << "Failed I2C reading ";
            return I2C_ERROR_FAILED_READING;
    }
    return SUCCESS;
}
