#include <iostream>
//#include <gtkmm-3.0/gtkmm.h>
#include <gtk/gtk.h>

#include "I2C_interface.h"
#include "SupportFunctions.h"

using namespace std;


// Custom structure that holds pointers to widgets and user variables
typedef struct {
    // Add pointers to widgets below
    GtkWidget *w_btn_zoom_in;                                // zoom controls
    GtkWidget *w_btn_zoom_out;
    GtkWidget *w_btn_zoom_stop;
    GtkWidget *w_comboboxtxt_zoom_speed;
    GtkWidget *w_comboboxtxt_zoom_mode;
    GtkWidget *w_scrb_zoom_pos;
    GtkWidget *w_lbl_zoom_pos_value;

    GtkWidget *w_btn_focus_tele;                        // focus controls
    GtkWidget *w_btn_focus_wide;
    GtkWidget *w_btn_focus_stop;
    GtkWidget *w_comboboxtxt_focus_speed;
    GtkWidget *w_comboboxtxt_focus_mode;

    GtkWidget *w_btn_wb_p2s;                            // white balance control
    GtkWidget *w_comboboxtxt_awb_mode;
    GtkWidget *w_lbl_man_red_gain;
    GtkWidget *w_scrb_man_red_gain;
    GtkWidget *w_lbl_man_blue_gain;
                                                                                                                                                                                                           GtkWidget *w_scrb_man_blue_gain;

   GtkWidget *w_chbtn_man_gain;                                     // AEX control
   GtkWidget *w_lbl_man_gain;
   GtkWidget *w_scrb_man_gain;
   GtkWidget *w_comboboxtxt_shutter_mode;


   GtkWidget *w_chbtn_man_iris;                                     // manual Iris control
   GtkWidget *w_lbl_man_iris;
   GtkWidget *w_scrb_man_iris;

   GtkWidget *w_chbtn_wdr_mode;
   GtkWidget *w_lbl_wdr_mode;
   GtkWidget *w_scrb_wdr_level;
   GtkWidget *w_chbtn_dnr_mode;
   GtkWidget *w_lbl_dnr_mode;
   GtkWidget *w_scrb_dnr_level;

   GtkWidget *w_comboboxtxt_is_level;
   GtkWidget *w_comboboxtxt_is_filter;

   GtkWidget *w_comboboxtxt_aperature_gain;
   GtkWidget *w_chbtn_mirror_mode;
   GtkWidget *w_chbtn_flip_mode;
   GtkWidget *w_chbtn_freeze;
   GtkWidget *w_chbtn_image_stabilizer_on;
   GtkWidget *w_comboboxtxt_video_mode;

   GtkWidget *w_btn_start_viewer;
   GtkWidget *w_btn_stop_viewer;
   GtkWidget *w_btn_snapshot;

} app_widgets;

void Set_Gui_controls_ZoomMode(app_widgets *app_wdgts);
void Set_Gui_controls_FocusMode(app_widgets *app_wdgts);
void LoadStartValues(void);


uint8_t zoomPos, zoomSpeed, zoomMode, focusMode, focusSpeed;
uint8_t manualgain, manualShutter, manualIris;
uint8_t wdrLevel, dnrLevel, stableizerLevel, stableizerFilter;
uint8_t manRedGain, manBlueGain;
uint8_t aexModus, awbModus, shutterMode;
uint8_t aperatureGain, videoMode;


int main(int argc, char *argv[])
{
     GtkBuilder      *builder;
    GtkWidget       *window;
    // Instantiate structure, allocating memory for it
    app_widgets     *widgets = g_slice_new(app_widgets);

    gtk_init(&argc, &argv);
    LoadStartValues();                                                    // default camera values-> later revision load from camera

    builder = gtk_builder_new_from_file("glade/window_main.glade");
    window = GTK_WIDGET(gtk_builder_get_object(builder, "window_main"));

    // Get pointers to widgets here
    widgets->w_btn_zoom_in = GTK_WIDGET(gtk_builder_get_object(builder, "btn_zoom_in"));// zoom controls
    widgets->w_btn_zoom_out = GTK_WIDGET(gtk_builder_get_object(builder, "btn_zoom_out"));
    widgets->w_btn_zoom_stop = GTK_WIDGET(gtk_builder_get_object(builder, "btn_zoom_stop"));

    widgets->w_btn_focus_stop = GTK_WIDGET(gtk_builder_get_object(builder, "btn_focus_stop"));// zoom controls
    widgets->w_btn_focus_tele = GTK_WIDGET(gtk_builder_get_object(builder, "btn_focus_tele"));
    widgets->w_btn_focus_wide= GTK_WIDGET(gtk_builder_get_object(builder, "btn_focus_wide"));

    widgets->w_comboboxtxt_zoom_speed  = GTK_WIDGET(gtk_builder_get_object(builder, "comboboxtxt_zoom_speed"));
    widgets->w_comboboxtxt_zoom_mode  = GTK_WIDGET(gtk_builder_get_object(builder, "comboboxtxt_zoom_mode"));
    widgets->w_comboboxtxt_focus_speed  = GTK_WIDGET(gtk_builder_get_object(builder, "comboboxtxt_focus_speed"));
    widgets->w_comboboxtxt_focus_mode = GTK_WIDGET(gtk_builder_get_object(builder, "comboboxtxt_focus_mode"));
    widgets->w_comboboxtxt_awb_mode = GTK_WIDGET(gtk_builder_get_object(builder, "comboboxtxt_awb_mode"));
    widgets->w_comboboxtxt_shutter_mode = GTK_WIDGET(gtk_builder_get_object(builder, "comboboxtxt_shutter_mode"));
    widgets->w_comboboxtxt_is_level = GTK_WIDGET(gtk_builder_get_object(builder, "comboboxtxt_is_level"));
    widgets->w_comboboxtxt_is_filter = GTK_WIDGET(gtk_builder_get_object(builder, "comboboxtxt_is_filter"));
    widgets->w_comboboxtxt_aperature_gain = GTK_WIDGET(gtk_builder_get_object(builder, "comboboxtxt_aperature_gain"));
    widgets->w_comboboxtxt_video_mode = GTK_WIDGET(gtk_builder_get_object(builder, "comboboxtxt_video_mode"));
    widgets->w_scrb_zoom_pos = GTK_WIDGET(gtk_builder_get_object(builder, "scrb_zoom_pos"));
    widgets->w_scrb_man_red_gain = GTK_WIDGET(gtk_builder_get_object(builder, "scrb_man_red_gain"));
    widgets->w_scrb_man_blue_gain = GTK_WIDGET(gtk_builder_get_object(builder, "scrb_man_blue_gain"));
    widgets->w_lbl_zoom_pos_value = GTK_WIDGET(gtk_builder_get_object(builder, "lbl_zoom_pos_value"));
    widgets->w_lbl_man_red_gain = GTK_WIDGET(gtk_builder_get_object(builder, "lbl_man_red_gain"));
    widgets->w_lbl_man_blue_gain = GTK_WIDGET(gtk_builder_get_object(builder, "lbl_man_blue_gain"));
    widgets->w_lbl_man_gain = GTK_WIDGET(gtk_builder_get_object(builder, "lbl_man_gain"));
    widgets->w_lbl_man_iris = GTK_WIDGET(gtk_builder_get_object(builder, "lbl_man_iris"));
    widgets->w_lbl_wdr_mode = GTK_WIDGET(gtk_builder_get_object(builder, "lbl_wdr_mode"));
    widgets->w_lbl_dnr_mode = GTK_WIDGET(gtk_builder_get_object(builder, "lbl_dnr_mode"));

    widgets->w_btn_start_viewer = GTK_WIDGET(gtk_builder_get_object(builder, "btn_start_viewer"));// zoom controls
    widgets->w_btn_stop_viewer = GTK_WIDGET(gtk_builder_get_object(builder, "btn_stop_viewer"));
    widgets->w_btn_snapshot = GTK_WIDGET(gtk_builder_get_object(builder, "btn_snapshot"));
   // widgets->w_w_btn_zoom_out  = GTK_WIDGET(gtk_builder_get_object(builder, "x"));
     // Widgets pointer are passed to all widget handler functions as the user_data parameter
    gtk_builder_connect_signals(builder, widgets);

    g_object_unref(builder);

    gtk_widget_show_all(window);
    gtk_main();
    // Free up widget structure memory
    g_slice_free(app_widgets, widgets);

    return 0;

}


void LoadStartValues(void)
{
//uint8_t errorMessage = SUCCESS;
uint8_t dataArray[NUMBER_OF_DEFAULT_VAL] = {0};
LoadDefaultValuesAtStartUp(dataArray);
zoomPos = dataArray[0];
zoomSpeed = dataArray[1];
focusSpeed = dataArray[2];
manualgain = dataArray[3];
manualShutter = dataArray[4];
manualIris = dataArray[5];
wdrLevel = dataArray[6];
dnrLevel = dataArray[7];
manRedGain = dataArray[8];
manBlueGain = dataArray[9];
stableizerLevel = dataArray[10];
stableizerFilter  = dataArray[11];
aperatureGain = dataArray[12];
videoMode = dataArray[13];
aexModus = 0x30;                    // default auto mode
awbModus = 0x20;
}
//************************************************************************************************************************
//  call functions
//************************************************************************************************************************

//--------------------------        Zoom buttons        ------------------------------------------------------------------
void on_btn_zoom_in_clicked()
{
    uint8_t *pZoomdata = 0;
    uint8_t errorMessage = I2C_Zoom_Fuction( I2C_ZOOM_IN, zoomSpeed, pZoomdata);
    if(errorMessage != SUCCESS)
    {
        g_print(" An error occured in Zoom-In call function %x: \n", errorMessage);
    }
}

void on_btn_zoom_out_clicked()
{
    uint8_t *pZoomdata = 0;
    uint8_t errorMessage = I2C_Zoom_Fuction( I2C_ZOOM_OUT, zoomSpeed, pZoomdata);
    if(errorMessage != SUCCESS)
    {
        g_print(" An error occured in Zoom-Out call function %x: \n", errorMessage);
    }
}

void on_btn_zoom_stop_clicked()
{
    uint8_t *pZoomdata = 0;
    uint8_t errorMessage = I2C_Zoom_Fuction( I2C_ZOOM_STOP, zoomSpeed, pZoomdata);
    if(errorMessage != SUCCESS)
    {
        g_print(" An error occured in Zoom-Stop call function %x: \n", errorMessage);
    }
}


//----------------------------   ComboBox controls   ---------------------------------------------------------------------

void on_comboboxtxt_zoom_speed_changed(GtkComboBox *widget,  app_widgets *app_wdgts)
{
    uint8_t errorMessage = SUCCESS;
    gint item_number = 0;
    item_number = gtk_combo_box_get_active(GTK_COMBO_BOX(app_wdgts->w_comboboxtxt_zoom_speed));
    zoomSpeed = (uint8_t) item_number;

}

void on_comboboxtxt_zoom_mode_changed(GtkComboBox *widget,  app_widgets *app_wdgts)
{
    uint8_t errorMessage = SUCCESS;
    gint item_number = 0;
    item_number = gtk_combo_box_get_active(GTK_COMBO_BOX(app_wdgts->w_comboboxtxt_zoom_mode));
    zoomMode = (uint8_t) item_number;
    Set_Gui_controls_ZoomMode(app_wdgts);

}

void Set_Gui_controls_ZoomMode(app_widgets *app_wdgts)
{
    switch (zoomMode){
        case 0:  gtk_widget_set_sensitive(app_wdgts->w_scrb_zoom_pos, FALSE); gtk_widget_set_sensitive(app_wdgts->w_btn_zoom_in, TRUE);gtk_widget_set_sensitive(app_wdgts->w_btn_zoom_out, TRUE);
                 gtk_widget_set_sensitive(app_wdgts->w_btn_zoom_stop, TRUE);gtk_widget_set_sensitive(app_wdgts->w_comboboxtxt_zoom_speed, TRUE);  break;
        case 1:  gtk_widget_set_sensitive(app_wdgts->w_scrb_zoom_pos, FALSE); gtk_widget_set_sensitive(app_wdgts->w_btn_zoom_in, TRUE);gtk_widget_set_sensitive(app_wdgts->w_btn_zoom_out, TRUE);
                 gtk_widget_set_sensitive(app_wdgts->w_btn_zoom_stop, TRUE);gtk_widget_set_sensitive(app_wdgts->w_comboboxtxt_zoom_speed, TRUE);  break;
        case 2:  gtk_widget_set_sensitive(app_wdgts->w_scrb_zoom_pos, TRUE); gtk_widget_set_sensitive(app_wdgts->w_comboboxtxt_zoom_speed,FALSE);
                 gtk_widget_set_sensitive(app_wdgts->w_btn_zoom_in,FALSE);gtk_widget_set_sensitive(app_wdgts->w_btn_zoom_out, FALSE);
                 gtk_widget_set_sensitive(app_wdgts->w_btn_zoom_stop, FALSE);break;
    }
}


//--------------------------        Focus buttons        ------------------------------------------------------------------

void on_btn_focus_tele_clicked()
{
    uint8_t *pZoomdata = 0;
    uint8_t errorMessage = I2C_Zoom_Fuction( I2C_ZOOM_IN, zoomSpeed, pZoomdata);
    if(errorMessage != SUCCESS)
    {
        g_print(" An error occured in Zoom-In call function %x: \n", errorMessage);
    }
}

void on_btn_focus_wide_clicked()
{
    uint8_t *pZoomdata = 0;
    uint8_t errorMessage = I2C_Zoom_Fuction( I2C_ZOOM_OUT, zoomSpeed, pZoomdata);
    if(errorMessage != SUCCESS)
    {
        g_print(" An error occured in Zoom-Out call function %x: \n", errorMessage);
    }
}

void on_btn_focus_stop_clicked()
{
    uint8_t *pZoomdata = 0;
    uint8_t errorMessage = I2C_Zoom_Fuction( I2C_ZOOM_STOP, zoomSpeed, pZoomdata);
    if(errorMessage != SUCCESS)
    {
        g_print(" An error occured in Zoom-Stop call function %x: \n", errorMessage);
    }
}

void on_comboboxtxt_focus_speed_changed(GtkComboBox *widget,  app_widgets *app_wdgts)
{
    gint item_number = 0;
   item_number = gtk_combo_box_get_active(GTK_COMBO_BOX(app_wdgts->w_comboboxtxt_focus_speed));

    focusSpeed = (uint8_t) item_number;
}

void on_comboboxtxt_focus_mode_changed(GtkComboBox *widget,  app_widgets *app_wdgts)
{
    uint8_t errorMessage = SUCCESS;
    gint item_number = 0;
    item_number = gtk_combo_box_get_active(GTK_COMBO_BOX(app_wdgts->w_comboboxtxt_focus_mode));
    focusMode = (uint8_t) item_number;
    Set_Gui_controls_FocusMode(app_wdgts);
}

void Set_Gui_controls_FocusMode(app_widgets *app_wdgts)
{
    switch (focusMode){
        case 0:  gtk_widget_set_sensitive(app_wdgts->w_btn_focus_tele, FALSE);gtk_widget_set_sensitive(app_wdgts->w_btn_focus_wide, FALSE);
                 gtk_widget_set_sensitive(app_wdgts->w_btn_focus_stop, FALSE);gtk_widget_set_sensitive(app_wdgts->w_comboboxtxt_focus_speed, FALSE);  break;
        case 1:  gtk_widget_set_sensitive(app_wdgts->w_btn_focus_tele, TRUE);gtk_widget_set_sensitive(app_wdgts->w_btn_focus_wide, TRUE);
                 gtk_widget_set_sensitive(app_wdgts->w_btn_focus_stop, TRUE);gtk_widget_set_sensitive(app_wdgts->w_comboboxtxt_focus_speed, TRUE);  break;
        case 2:  gtk_widget_set_sensitive(app_wdgts->w_btn_focus_tele, FALSE);gtk_widget_set_sensitive(app_wdgts->w_btn_focus_wide, FALSE);
                 gtk_widget_set_sensitive(app_wdgts->w_btn_focus_stop, FALSE);gtk_widget_set_sensitive(app_wdgts->w_comboboxtxt_focus_speed, FALSE);  break;
    }
}
//--------------------------        AWB buttons        ------------------------------------------------------------------
void on_btn_wb_p2s_clicked()
{
    uint8_t *pZoomdata = 0;
    uint8_t errorMessage = I2C_Zoom_Fuction( I2C_ZOOM_STOP, zoomSpeed, pZoomdata);
    if(errorMessage != SUCCESS)
    {
        g_print(" An error occured in Zoom-Stop call function %x: \n", errorMessage);
    }
}


//************************************************************************************************************************
// check buttons
//************************************************************************************************************************

void on_chbtn_man_gain_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
	//g_print("%s\n", gtk_widget_get_name(GTK_WIDGET(togglebutton)));
	uint8_t errorMessage = SUCCESS;
	uint8_t pMangain[1];
	pMangain[0]= manualgain ;
	if (gtk_toggle_button_get_active(togglebutton)) aexModus |= AEX_GAIN_MANUAL ;       // set manual gain flag
	else aexModus &= ~AEX_GAIN_MANUAL ;                                                  // clear manual gain
	errorMessage = I2C_AEX_Fuction( aexModus, pMangain);

}

void on_chbtn_man_iris_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
	//g_print("%s\n", gtk_widget_get_name(GTK_WIDGET(togglebutton)));
	uint8_t errorMessage = SUCCESS;
	uint8_t pManiris[1];
	pManiris[0]= manualgain ;
	if (gtk_toggle_button_get_active(togglebutton)) aexModus |= AEX_IRIS_MANUAL ;       // set manual gain flag
	else aexModus &= ~AEX_IRIS_MANUAL ;                                                  // clear manual gain
	errorMessage = I2C_AEX_Fuction( aexModus, pManiris);
}

void on_chbtn_man_wdr_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
	uint8_t errorMessage = SUCCESS;
	uint8_t mode;
	if (gtk_toggle_button_get_active(togglebutton)) mode = WDR_MODE_ON;      // set manual gain flag
	else mode = WDR_MODE_OFF;                                                // clear manual gain
	errorMessage = I2C_WDR_Fuction( mode, wdrLevel);
}

void on_chbtn_man_dnr_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
	uint8_t errorMessage = SUCCESS;
	uint8_t mode;
	if (gtk_toggle_button_get_active(togglebutton)) mode = DNR_MODE_ON;      // set manual gain flag
	else mode = DNR_MODE_OFF;                                                // clear manual gain
	errorMessage = I2C_DNR_Fuction( mode, wdrLevel);
}

void on_chbtn_mirror_mode_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
    uint8_t errorMessage = SUCCESS;
    uint8_t pData[1];
    if (gtk_toggle_button_get_active(togglebutton)) pData[0] = 1;      // set manual gain flag
	else pData[0] = 0;
    I2C_Feature_Fuction( I2C_MIRROR_MODE, pData);
}

void on_chbtn_flip_mode_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
    uint8_t errorMessage = SUCCESS;
    uint8_t pData[1];
    if (gtk_toggle_button_get_active(togglebutton)) pData[0] = 1;      // set manual gain flag
	else pData[0] = 0;
    I2C_Feature_Fuction( I2C_FLIP_MODE, pData);
}

void on_chbtn_freeze_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
    uint8_t errorMessage = SUCCESS;
    uint8_t pData[1];
    if (gtk_toggle_button_get_active(togglebutton)) pData[0] = 1;      // set manual gain flag
	else pData[0] = 0;
    I2C_Feature_Fuction( I2C_FREEZE_MODE, pData);
}

void on_chbtn_image_stabilizer_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
    uint8_t errorMessage = SUCCESS;
    uint8_t pData[2];
    if (gtk_toggle_button_get_active(togglebutton))
    {
        pData[0] = 1;      // set manual gain flag
        pData[1] = stableizerLevel;
    }
	else pData[0] = 0;
    I2C_Feature_Fuction( I2C_IMAGE_STABALIZER, pData);
}



//************************************************************************************************************************
// ComboBox controls
//************************************************************************************************************************

void on_comboboxtxt_awb_mode_changed(GtkComboBox *widget,  app_widgets *app_wdgts)
{
    gint item_number = 0;
    item_number = gtk_combo_box_get_active(GTK_COMBO_BOX(app_wdgts->w_comboboxtxt_awb_mode));
    awbModus = (uint8_t) item_number;
}

void on_comboboxtxt_shutter_mode_changed(GtkComboBox *widget,  app_widgets *app_wdgts)
{
    gint item_number = 0;
    item_number = gtk_combo_box_get_active(GTK_COMBO_BOX(app_wdgts->w_comboboxtxt_zoom_speed));
    manualShutter = (uint8_t) item_number;
}

void on_comboboxtxt_is_level_changed(GtkComboBox *widget,  app_widgets *app_wdgts)
{
    gint item_number = 0;
    item_number = gtk_combo_box_get_active(GTK_COMBO_BOX(app_wdgts->w_comboboxtxt_is_level));
    stableizerLevel = (uint8_t) item_number;
}

void on_comboboxtxt_is_filter_changed(GtkComboBox *widget,  app_widgets *app_wdgts)
{
    gint item_number = 0;
    item_number = gtk_combo_box_get_active(GTK_COMBO_BOX(app_wdgts->w_comboboxtxt_is_filter));
    stableizerFilter = (uint8_t) item_number;
}

void on_comboboxtxt_aperature_gain_changed(GtkComboBox *widget,  app_widgets *app_wdgts)
{
    gint item_number = 0;
    item_number = gtk_combo_box_get_active(GTK_COMBO_BOX(app_wdgts->w_comboboxtxt_aperature_gain));
    aperatureGain = (uint8_t) item_number;
}

void on_comboboxtxt_video_mode_changed(GtkComboBox *widget,  app_widgets *app_wdgts)
{
    gint item_number = 0;
    item_number = gtk_combo_box_get_active(GTK_COMBO_BOX(app_wdgts->w_comboboxtxt_video_mode));
    videoMode = (uint8_t) item_number;
}
// Dummy handler function
/*void x(GtkButton *button, app_widgets *app_wdgts)
{
}*/

//*********************************************************************************************************************************************************
// slider controls
//********************************************************************************************************************************************************
void on_scrb_zoom_pos_value_changed(GtkRange *range,  app_widgets *app_wdgts)
{
    uint8_t errorMessage = SUCCESS;
    gdouble scbValue;
    gchar zoom_pos_str[5] = {0}; // buffer for string
    scbValue = gtk_range_get_value(range);
    ConvertUint2string((unsigned int) scbValue, zoom_pos_str);
    gtk_label_set_text(GTK_LABEL(app_wdgts->w_lbl_zoom_pos_value), zoom_pos_str);
    zoomPos = (uint8_t)scbValue;

    errorMessage = I2C_Zoom_Fuction( I2C_ZOOM_IN, zoomSpeed, &zoomPos);
     //valueSB = gtk_scrollable_get_  _hadjustment (GTK_COMBO_BOX(app_wdgts->w_scrb_zoom_pos));
}

void on_scrb_man_red_gain_value_changed(GtkRange *range,  app_widgets *app_wdgts)
{
    gdouble scbValue;
    gchar gain_pos_str[5] = {0}; // buffer for string
    scbValue = gtk_range_get_value(range);
    ConvertUint2string((unsigned int) scbValue, gain_pos_str);
    gtk_label_set_text(GTK_LABEL(app_wdgts->w_lbl_man_red_gain), gain_pos_str);
}

void on_scrb_man_blue_gain_value_changed(GtkRange *range,  app_widgets *app_wdgts)
{
    gdouble scbValue;
    gchar gain_pos_str[5] = {0}; // buffer for string
    scbValue = gtk_range_get_value(range);
    ConvertUint2string((unsigned int) scbValue, gain_pos_str);
    gtk_label_set_text(GTK_LABEL(app_wdgts->w_lbl_man_blue_gain), gain_pos_str);
}

void on_scrb_man_gain_value_changed(GtkRange *range,  app_widgets *app_wdgts)
{
    gdouble scbValue;
    gchar gain_pos_str[5] = {0}; // buffer for string
    scbValue = gtk_range_get_value(range);
    ConvertUint2string((unsigned int) scbValue, gain_pos_str);
    gtk_label_set_text(GTK_LABEL(app_wdgts->w_lbl_man_gain), gain_pos_str);
}

void on_scrb_man_iris_value_changed(GtkRange *range,  app_widgets *app_wdgts)
{
    gdouble scbValue;
    gchar gain_pos_str[5] = {0}; // buffer for string
    scbValue = gtk_range_get_value(range);
    ConvertUint2string((unsigned int) scbValue, gain_pos_str);
    gtk_label_set_text(GTK_LABEL(app_wdgts->w_lbl_man_iris), gain_pos_str);
}

void on_scrb_wdr_level_value_changed(GtkRange *range,  app_widgets *app_wdgts)
{
    gdouble scbValue;
    gchar gain_pos_str[5] = {0}; // buffer for string
    scbValue = gtk_range_get_value(range);
    ConvertUint2string((unsigned int) scbValue, gain_pos_str);
    gtk_label_set_text(GTK_LABEL(app_wdgts->w_lbl_wdr_mode), gain_pos_str);
}

void on_scrb_dnr_level_value_changed(GtkRange *range,  app_widgets *app_wdgts)
{
    gdouble scbValue;
    gchar gain_pos_str[5] = {0}; // buffer for string
    scbValue = gtk_range_get_value(range);
    ConvertUint2string((unsigned int) scbValue, gain_pos_str);
    gtk_label_set_text(GTK_LABEL(app_wdgts->w_lbl_dnr_mode), gain_pos_str);
}

void on_btn_start_viewer_clicked(GtkButton *button, app_widgets *app_wdgts)
{
    gtk_widget_set_sensitive(app_wdgts->w_btn_stop_viewer, TRUE);       // make stop viewer button active
    gtk_widget_set_sensitive(app_wdgts->w_btn_snapshot, TRUE);       // make snapshot button active
    CreateThread();
    gtk_widget_set_sensitive(app_wdgts->w_btn_start_viewer, FALSE);       // make stop viewer button active
}

void on_btn_stop_viewer_clicked(GtkButton *button,app_widgets *app_wdgts)
{
    gtk_widget_set_sensitive(app_wdgts->w_btn_stop_viewer, FALSE);       // make stop viewer button active
    gtk_widget_set_sensitive(app_wdgts->w_btn_snapshot, FALSE);       // make snapshot button active
    stopMyThreadProc();
    gtk_widget_set_sensitive(app_wdgts->w_btn_start_viewer, TRUE);       // make stop viewer button active
}

void on_btn_snapshot_clicked()
{

}

//********************************************************************************************************************************************************
// close window function
//********************************************************************************************************************************************************
// called when window is closed
void on_window_main_destroy()
{
    gtk_main_quit();
}

